=== modified file 'client/drizzle.cc'
--- client/drizzle.cc	2010-10-30 01:19:00 +0000
+++ client/drizzle.cc	2010-11-07 12:27:27 +0000
@@ -2319,6 +2319,12 @@
   {
     *out++='\n';
     uint32_t length=(uint32_t) (out-line);
+    if ((buffer->length() + length) > opt_max_input_line)
+    {
+      status.setExitStatus(1);
+      put_info(_("Not found a delimiter within max_input_line of input"), INFO_ERROR, 0, 0);
+      return 1;
+    }
     if ((!*ml_comment || preserve_comments))
       buffer->append(line, length);
   }

=== modified file 'client/linebuffer.cc'
--- client/linebuffer.cc	2010-10-02 21:15:42 +0000
+++ client/linebuffer.cc	2010-11-07 12:27:27 +0000
@@ -27,36 +27,25 @@
 LineBuffer::LineBuffer(uint32_t my_max_size,FILE *my_file)
   :
     file(my_file),
-    line(),
-    max_size(my_max_size),
-    eof(false)
+    max_size(my_max_size)
 {
+  if (my_file)
+    file_stream = new boost::iostreams::stream<boost::iostreams::file_descriptor>(fileno(my_file), true);
+  else
+    file_stream = new std::stringstream;
   line.reserve(max_size);
 }
 
 void LineBuffer::addString(const string &str)
 {
-  buffer << str << endl;
+  (*file_stream) << str << endl;
 }
 
 char *LineBuffer::readline()
 {
-  uint32_t read_count;
-
-  if (file && !eof)
-  {
-    if ((read_count= read(fileno(file), (&line[0]), max_size-1)))
-    {
-      line[read_count+1]= '\0';
-      buffer << &line[0];
-    }
-    else
-      eof= true;
-  }
-
-  buffer.getline(&line[0],max_size);
-
-  if (buffer.eof())
+  file_stream->getline(&line[0], max_size);
+
+  if (file_stream->fail())
     return 0;
   else
     return &line[0];

=== modified file 'client/linebuffer.h'
--- client/linebuffer.h	2010-10-02 21:15:42 +0000
+++ client/linebuffer.h	2010-11-07 12:27:27 +0000
@@ -18,20 +18,21 @@
 
 #include <vector>
 #include <sstream>
+#include <iostream>
+#include <boost/iostreams/stream.hpp>
+#include <boost/iostreams/device/file_descriptor.hpp>
 
 class LineBuffer
 {
 public:
   LineBuffer(uint32_t max_size,FILE *file);
-
   void addString(const std::string &argument);
   char *readline();
 private:
   FILE *file;
-  std::stringstream buffer;
   std::vector<char> line;
+  std::iostream *file_stream;
   uint32_t max_size;
-  bool eof;
 };
 
 #endif /* CLIENT_LINEBUFFER_H */

=== modified file 'm4/pandora_drizzle_build.m4'
--- m4/pandora_drizzle_build.m4	2010-10-08 17:32:00 +0000
+++ m4/pandora_drizzle_build.m4	2010-11-07 12:27:27 +0000
@@ -72,5 +72,6 @@
   PANDORA_REQUIRE_BOOST_REGEX
   PANDORA_REQUIRE_BOOST_DATE_TIME
   PANDORA_REQUIRE_BOOST_FILESYSTEM
+  PANDORA_REQUIRE_BOOST_IOSTREAMS
 
 ])

=== added file 'm4/pandora_have_libboost_iostreams.m4'
--- m4/pandora_have_libboost_iostreams.m4	1970-01-01 00:00:00 +0000
+++ m4/pandora_have_libboost_iostreams.m4	2010-11-07 12:27:27 +0000
@@ -0,0 +1,49 @@
+dnl Copyright (C) 2010 Andrew Hutchings
+dnl This file is free software; Andrew Hutchings
+dnl gives unlimited permission to copy and/or distribute it,
+dnl with or without modifications, as long as this notice is preserved.
+
+AC_DEFUN([_PANDORA_SEARCH_BOOST_IOSTREAMS],[
+  AC_REQUIRE([AC_LIB_PREFIX])
+
+  dnl --------------------------------------------------------------------
+  dnl  Check for Boost.Filesystem
+  dnl --------------------------------------------------------------------
+
+  AC_LANG_PUSH(C++)
+  AC_LIB_HAVE_LINKFLAGS(boost_iostreams-mt,,[
+    #include <boost/iostreams/stream.hpp>
+    #include <boost/iostreams/device/array.hpp>
+  ],[
+    const char* input= "hello world";
+    boost::iostreams::stream<boost::iostreams::array_source> in(input, strlen(input));
+  ])
+  AS_IF([test "x${ac_cv_libboost_iostreams_mt}" = "xno"],[
+    AC_LIB_HAVE_LINKFLAGS(boost_iostreams,,[
+      #include <boost/iostreams/stream.hpp>
+      #include <boost/iostreams/device/array.hpp>
+    ],[
+      const char* input= "hello world";
+      boost::iostreams::stream<boost::iostreams::array_source> in(input, strlen(input));
+    ])
+  ])
+  AC_LANG_POP()
+  
+  AM_CONDITIONAL(HAVE_BOOST_IOSTREAMS,
+    [test "x${ac_cv_libboost_iostreams}" = "xyes" -o "x${ac_cv_libboost_iostreams_mt}" = "xyes"])
+  BOOST_LIBS="${BOOST_LIBS} ${LTLIBBOOST_IOSTREAMS_MT} ${LTLIBBOOST_IOSTREAMS}"
+  AC_SUBST(BOOST_LIBS) 
+])
+
+AC_DEFUN([PANDORA_HAVE_BOOST_IOSTREAMS],[
+  PANDORA_HAVE_BOOST($1)
+  _PANDORA_SEARCH_BOOST_IOSTREAMS($1)
+])
+
+AC_DEFUN([PANDORA_REQUIRE_BOOST_IOSTREAMS],[
+  PANDORA_REQUIRE_BOOST($1)
+  _PANDORA_SEARCH_BOOST_IOSTREAMS($1)
+  AS_IF([test "x${ac_cv_libboost_iostreams}" = "xno" -a "x${ac_cv_libboost_iostreams_mt}" = "xno"],
+      AC_MSG_ERROR([Boost.Iostreams is required for ${PACKAGE}]))
+])
+

